@@ -1,7 +1,8 @@
 from collections import Iterable
+import six
 from pyrsistent._checked_types import (
     CheckedType, CheckedPSet, CheckedPMap, CheckedPVector,
-    optional as optional_type, InvariantException)
+    optional as optional_type, InvariantException, get_type, wrap_invariant)
 
 
 def set_fields(dct, bases, name):
@@ -13,18 +14,11 @@ def set_fields(dct, bases, name):
             del dct[k]
 
 
-def set_global_invariants(dct, bases, name):
-    dct[name] = [dct['__invariant__']] if '__invariant__' in dct else []
-    dct[name] += [b.__dict__['__invariant__'] for b in bases if '__invariant__' in b.__dict__]
-    if not all(callable(invariant) for invariant in dct[name]):
-        raise TypeError('Global invariants must be callable')
-
-
 def check_global_invariants(subject, invariants):
-        error_codes = tuple(error_code for is_ok, error_code in
-                            (invariant(subject) for invariant in invariants) if not is_ok)
-        if error_codes:
-            raise InvariantException(error_codes, (), 'Global invariant failed')
+    error_codes = tuple(error_code for is_ok, error_code in
+                        (invariant(subject) for invariant in invariants) if not is_ok)
+    if error_codes:
+        raise InvariantException(error_codes, (), 'Global invariant failed')
 
 
 def serialize(serializer, format, value):
@@ -35,29 +29,39 @@ def serialize(serializer, format, value):
 
 
 def check_type(destination_cls, field, name, value):
-    if field.type and not any(isinstance(value, t) for t in field.type):
+    if field.type and not any(isinstance(value, get_type(t)) for t in field.type):
         actual_type = type(value)
         message = "Invalid type for field {0}.{1}, was {2}".format(destination_cls.__name__, name, actual_type.__name__)
         raise PTypeError(destination_cls, name, field.type, actual_type, message)
 
 
 class _PField(object):
-    __slots__ = ('type', 'invariant', 'initial', 'mandatory', 'factory', 'serializer')
+    __slots__ = ('type', 'invariant', 'initial', 'mandatory', '_factory', 'serializer')
 
     def __init__(self, type, invariant, initial, mandatory, factory, serializer):
         self.type = type
         self.invariant = invariant
         self.initial = initial
         self.mandatory = mandatory
-        self.factory = factory
+        self._factory = factory
         self.serializer = serializer
 
+    @property
+    def factory(self):
+        # If no factory is specified and the type is another CheckedType use the factory method of that CheckedType
+        if self._factory is PFIELD_NO_FACTORY and len(self.type) == 1:
+            typ = get_type(tuple(self.type)[0])
+            if issubclass(typ, CheckedType):
+                return typ.create
+
+        return self._factory
+
 PFIELD_NO_TYPE = ()
 PFIELD_NO_INVARIANT = lambda _: (True, None)
 PFIELD_NO_FACTORY = lambda x: x
 PFIELD_NO_INITIAL = object()
 PFIELD_NO_SERIALIZER = lambda _, value: value
-
+PFIELD_NO_SERIALIZER = lambda _, value: value
 
 def field(type=PFIELD_NO_TYPE, invariant=PFIELD_NO_INVARIANT, initial=PFIELD_NO_INITIAL,
           mandatory=False, factory=PFIELD_NO_FACTORY, serializer=PFIELD_NO_SERIALIZER):
@@ -67,22 +71,14 @@ def field(type=PFIELD_NO_TYPE, invariant=PFIELD_NO_INVARIANT, initial=PFIELD_NO_
     :param type: a type or iterable with types that are allowed for this field
     :param invariant: a function specifying an invariant that must hold for the field
     :param initial: value of field if not specified when instantiating the record
-    :param mandatorty: boolean specifying if the field is mandatory or not
+    :param mandatory: boolean specifying if the field is mandatory or not
     :param factory: function called when field is set.
     :param serializer: function that returns a serialized version of the field
     """
-
-    types = set(type) if isinstance(type, Iterable) else set([type])
-
-    # If no factory is specified and the type is another CheckedType use the factory method of that CheckedType
-    if factory is PFIELD_NO_FACTORY and len(types) == 1 and issubclass(tuple(types)[0], CheckedType):
-        # TODO: Should this option be looked up at execution time rather than at field construction time?
-        #       that would allow checking against all the types specified and if none matches the
-        #       first
-        factory = tuple(types)[0].create
-
-    field = _PField(type=types, invariant=invariant, initial=initial, mandatory=mandatory,
-                    factory=factory, serializer=serializer)
+    types = set(type) if isinstance(type, Iterable) and not isinstance(type, six.string_types) else set([type])
+    invariant_function = wrap_invariant(invariant) if invariant != PFIELD_NO_INVARIANT and callable(invariant) else invariant
+    field = _PField(type=types, invariant=invariant_function, initial=initial,
+                    mandatory=mandatory, factory=factory, serializer=serializer)
 
     _check_field_parameters(field)
 
@@ -91,8 +87,8 @@ def field(type=PFIELD_NO_TYPE, invariant=PFIELD_NO_INVARIANT, initial=PFIELD_NO_
 
 def _check_field_parameters(field):
     for t in field.type:
-        if not isinstance(t, type):
-            raise TypeError('Type paramenter expected, not {0}'.format(type(t)))
+        if not isinstance(t, type) and not isinstance(t, six.string_types):
+            raise TypeError('Type parameter expected, not {0}'.format(type(t)))
 
     if field.initial is not PFIELD_NO_INITIAL and field.type and not any(isinstance(field.initial, t) for t in field.type):
         raise TypeError('Initial has invalid type {0}'.format(type(t)))
@@ -132,7 +128,7 @@ def _sequence_field(checked_class, suffix, item_type, optional, initial):
     :param checked_class: ``CheckedPSet`` or ``CheckedPVector``.
     :param suffix: Suffix for new type name.
     :param item_type: The required type for the items in the set.
-    :param bool optional: If true, ``None`` can be used as a value for
+    :param optional: If true, ``None`` can be used as a value for
         this field.
     :param initial: Initial value to pass to factory.
 
@@ -161,7 +157,7 @@ def pset_field(item_type, optional=False, initial=()):
     Create checked ``PSet`` field.
 
     :param item_type: The required type for the items in the set.
-    :param bool optional: If true, ``None`` can be used as a value for
+    :param optional: If true, ``None`` can be used as a value for
         this field.
     :param initial: Initial value to pass to factory if no value is given
         for the field.
@@ -177,7 +173,7 @@ def pvector_field(item_type, optional=False, initial=()):
     Create checked ``PVector`` field.
 
     :param item_type: The required type for the items in the vector.
-    :param bool optional: If true, ``None`` can be used as a value for
+    :param optional: If true, ``None`` can be used as a value for
         this field.
     :param initial: Initial value to pass to factory if no value is given
         for the field.
@@ -197,7 +193,7 @@ def pmap_field(key_type, value_type, optional=False, invariant=PFIELD_NO_INVARIA
 
     :param key: The required type for the keys of the map.
     :param value: The required type for the values of the map.
-    :param bool optional: If true, ``None`` can be used as a value for
+    :param optional: If true, ``None`` can be used as a value for
         this field.
     :param invariant: Pass-through to ``field``.
 
